////////////////////////////////////////////////////////////////////////////////
//
//  Licensed to the Apache Software Foundation (ASF) under one or more
//  contributor license agreements.  See the NOTICE file distributed with
//  this work for additional information regarding copyright ownership.
//  The ASF licenses this file to You under the Apache License, Version 2.0
//  (the "License"); you may not use this file except in compliance with
//  the License.  You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//
////////////////////////////////////////////////////////////////////////////////
package org.apache.royale.core
{
	import org.apache.royale.events.IEventDispatcher;

    /**
     *  The IItemRenderer interface is the basic interface for item renderers.
     *  More complex item renderers may implements extensions of this interface.
     *  
     *  @langversion 3.0
     *  @playerversion Flash 10.2
     *  @playerversion AIR 2.6
     *  @productversion Royale 0.0
     */
	public interface ISelectableItemRenderer extends IItemRenderer
	{
		
        /**
         *  The property on the data item that the item renderer
         *  should renderer.
         * 
         *  @langversion 3.0
         *  @playerversion Flash 10.2
         *  @playerversion AIR 2.6
         *  @productversion Royale 0.0
         */
		function get labelField():String;
		function set labelField(value:String):void;
		
        /**
         *  The index of the data item in the data provider.
         * 
         *  @langversion 3.0
         *  @playerversion Flash 10.2
         *  @playerversion AIR 2.6
         *  @productversion Royale 0.0
         */
		function get index():int;
		function set index(value:int):void;
		
        /**
         *  <code>true</code> if the item renderer is rendering
         *  an item that is selected.
         * 
         *  @langversion 3.0
         *  @playerversion Flash 10.2
         *  @playerversion AIR 2.6
         *  @productversion Royale 0.0
         */
		function get selected():Boolean;
		function set selected(value:Boolean):void;
        
        /**
         *  <code>true</code> if the item renderer is rendering
         *  an item that is under the mouse pointer.
         * 
         *  @langversion 3.0
         *  @playerversion Flash 10.2
         *  @playerversion AIR 2.6
         *  @productversion Royale 0.0
         */
        function get hovered():Boolean;
        function set hovered(value:Boolean):void;

        /**
         *  <code>true</code> if the item renderer is rendering
         *  an item that is under the mouse pointer and the
         *  mouse button is down.
         * 
         *  @langversion 3.0
         *  @playerversion Flash 10.2
         *  @playerversion AIR 2.6
         *  @productversion Royale 0.0
         */
        function get down():Boolean;
        function set down(value:Boolean):void;
	}
}
